@@ -1,4 +1,5 @@
 const nofreefns = Set{String}((
+    "jl_genericmemory_copyto",
     "utf8proc_toupper",
     "ClientGetAddressableDevices",
     "ClientNumAddressableDevices",
@@ -187,9 +188,33 @@ const nofreefns = Set{String}((
     "ijl_process_events",
     "utf8proc_category",
     "jl_clock_now",
+    "ijl_new_task",
+    "cuMemPoolGetAttribute",
+    "ijl_hrtime",
+    "ijl_gc_collect",
+    "uv_thread_detach",
+    "uv_thread_create",
+    "cuDeviceCanAccessPeer",
+    "cuMemPoolSetAccess",
+    "cuMemcpyHtoDAsync_v2",
+    "cuDeviceGet",
+    "cuDeviceGetCount",
+    "cuDeviceGetAttribute",
+    "cuDevicePrimaryCtxRetain",
+    "cuCtxGetId",
+    "cuDeviceGetName",
+    "ijl_eqtable_get",
+    "cuCtxGetApiVersion",
+    "cuCtxSetCurrent",
 ))
 
 const inactivefns = Set{String}((
+    
+    # The binding lookup is asserted as inactive [e.g. globals are considered constant wrt differentiation]
+    # If and when this changes, we need to change that here
+    "jl_get_binding_value_seqcst",
+    "ijl_get_binding_value_seqcst",
+
     "utf8proc_toupper",
     "ClientGetAddressableDevices",
     "ClientNumAddressableDevices",
