@@ -1947,7 +1947,17 @@ end
 end
 
 @register_fwd function get_binding_or_error_fwd(B, orig, gutils, normalR, shadowR)
-    if is_constant_value(gutils, orig)
+    needsShadowP = Ref{UInt8}(0)
+    needsPrimalP = Ref{UInt8}(0)
+    activep = API.EnzymeGradientUtilsGetReturnDiffeType(
+        gutils,
+        orig,
+        needsPrimalP,
+        needsShadowP,
+        get_mode(gutils),
+    )
+
+    if is_constant_value(gutils, orig) || needsShadowP[] == 0
         return true
     end
 
@@ -1989,48 +1999,24 @@ end
 end
 
 @register_aug function get_binding_or_error_augfwd(B, orig, gutils, normalR, shadowR, tapeR)
-    if is_constant_value(gutils, orig)
-        return true
-    end
-
-    newo = new_from_original(gutils, orig)
-
-    cmp = icmp!(B, LLVM.API.LLVMIntNE, newo, LLVM.null(value_type(newo)))
-
-    err = emit_error(
-        B,
-        orig,
-        "Enzyme: unhandled augmented forward for jl_get_binding_or_error",
-        EnzymeRuntimeException,
-        cmp
-    )
-    API.moveBefore(newo, cmp, B)
-
-    if unsafe_load(shadowR) != C_NULL
-	valTys = API.CValueType[]
-	args = LLVM.Value[]
-	for i in 1:(length(operands(orig))-1)    
-	    push!(valTys, API.VT_Primal)
-	    push!(args, new_from_original(gutils, operands(orig)[i]))
-	end
-	normal = call_samefunc_with_inverted_bundles!(B, gutils, orig, args, valTys, false) #=lookup=#
-        width = get_width(gutils)
-        if width == 1
-            shadowres = normal
-        else
-            shadowres = UndefValue(
-                LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(normal))),
-            )
-            for idx = 1:width
-                shadowres = insert_value!(B, shadowres, normal, idx - 1)
-            end
-        end
-        unsafe_store!(shadowR, shadowres.ref)
-    end
-    return false
+    return get_binding_or_error_fwd(B, orig, gutils, normalR, shadowR)
 end
 
 @register_rev function get_binding_or_error_rev(B, orig, gutils, tape)
+    needsShadowP = Ref{UInt8}(0)
+    needsPrimalP = Ref{UInt8}(0)
+    activep = API.EnzymeGradientUtilsGetReturnDiffeType(
+        gutils,
+        orig,
+        needsPrimalP,
+        needsShadowP,
+        API.DEM_ReverseModePrimal,
+    )
+
+    if is_constant_value(gutils, orig) || needsShadowP[] == 0
+        return nothing
+    end
+
     emit_error(B, orig, "Enzyme: unhandled reverse for jl_get_binding_or_error")
     return nothing
 end
