@@ -3,10 +3,10 @@ module BijectorsIntegrationTests
 using Bijectors: Bijectors
 using Enzyme: Enzyme
 using FiniteDifferences: FiniteDifferences
-using LinearAlgebra: LinearAlgebra
+using LinearAlgebra: I, cholesky, Hermitian
 using Random: randn
 using StableRNGs: StableRNG
-using Test: @test, @test_broken, @testset
+using Test: @test, @test_broken, @testset, @test_throws
 
 rng = StableRNG(23)
 
@@ -14,6 +14,8 @@ rng = StableRNG(23)
 Enum type for choosing Enzyme autodiff modes.
 """
 @enum ModeSelector Neither Forward Reverse Both
+includes_forward(mode::ModeSelector) = mode === Forward || mode === Both
+includes_reverse(mode::ModeSelector) = mode === Reverse || mode === Both
 
 """
 Type for specifying a test case for `Enzyme.gradient`.
@@ -42,41 +44,49 @@ end
 
 # Default values for most arguments.
 function TestCase(
-    f, value;
-    name=nothing, runtime_activity=Neither, broken=Neither, skip=Neither, splat=false
-)
+        f, value;
+        name = nothing, runtime_activity = Neither, broken = Neither, skip = Neither, splat = false
+    )
     return TestCase(f, value, name, runtime_activity, broken, skip, splat)
 end
 
 """
 Test Enzyme.gradient, both Forward and Reverse mode, against FiniteDifferences.grad.
 """
-function test_grad(case::TestCase; rtol=1e-6, atol=1e-6)
+function test_grad(case::TestCase; rtol = 1.0e-6, atol = 1.0e-6)
     @nospecialize
     f = case.func
     # We'll call the function as f(x...), so wrap in a singleton tuple if need be.
     x = case.splat ? case.value : (case.value,)
     finitediff = FiniteDifferences.grad(FiniteDifferences.central_fdm(4, 1), f, x...)[1]
 
-    f_mode = if (case.runtime_activity === Both || case.runtime_activity === Forward)
+    f_mode = if includes_forward(case.runtime_activity)
         Enzyme.set_runtime_activity(Enzyme.Forward)
     else
         Enzyme.Forward
     end
-    r_mode = if (case.runtime_activity === Both || case.runtime_activity === Reverse)
+    r_mode = if includes_reverse(case.runtime_activity)
         Enzyme.set_runtime_activity(Enzyme.Reverse)
     else
         Enzyme.Reverse
     end
 
-    if !(case.skip === Forward) && !(case.skip === Both)
-        if case.broken === Both || case.broken === Forward
+    if !includes_forward(case.skip)
+        if includes_forward(case.broken)
             @test_broken(
                 Enzyme.gradient(f_mode, Enzyme.Const(f), x...)[1] ≈ finitediff,
                 rtol = rtol,
                 atol = atol,
             )
         else
+            # If runtime activity was requested, check that it actually was needed.
+            if includes_forward(case.runtime_activity)
+                @test_throws Enzyme.Compiler.EnzymeRuntimeActivityError Enzyme.gradient(
+                    Enzyme.Forward,
+                    Enzyme.Const(f),
+                    x...,
+                )
+            end
             @test(
                 Enzyme.gradient(f_mode, Enzyme.Const(f), x...)[1] ≈ finitediff,
                 rtol = rtol,
@@ -85,14 +95,22 @@ function test_grad(case::TestCase; rtol=1e-6, atol=1e-6)
         end
     end
 
-    if !(case.skip === Reverse) && !(case.skip === Both)
-        if case.broken === Both || case.broken === Reverse
+    if !includes_reverse(case.skip)
+        if includes_reverse(case.broken)
             @test_broken(
                 Enzyme.gradient(r_mode, Enzyme.Const(f), x...)[1] ≈ finitediff,
                 rtol = rtol,
                 atol = atol,
             )
         else
+            # If runtime activity was requested, check that it actually was needed.
+            if includes_reverse(case.runtime_activity)
+                @test_throws Enzyme.Compiler.EnzymeRuntimeActivityError Enzyme.gradient(
+                    Enzyme.Reverse,
+                    Enzyme.Const(f),
+                    x...,
+                )
+            end
             @test(
                 Enzyme.gradient(r_mode, Enzyme.Const(f), x...)[1] ≈ finitediff,
                 rtol = rtol,
@@ -107,8 +125,8 @@ end
 A helper function that returns a TestCase that evaluates sum(bijector(inverse(bijector)(x)))
 """
 function sum_b_binv_test_case(
-    bijector, dim; runtime_activity=Neither, name=nothing, broken=Neither, skip=Neither
-)
+        bijector, dim; runtime_activity = Neither, name = nothing, broken = Neither, skip = Neither
+    )
     if name === nothing
         name = string(bijector)
     end
@@ -116,13 +134,14 @@ function sum_b_binv_test_case(
     return TestCase(
         x -> sum(bijector(b_inv(x))),
         randn(rng, dim);
-        runtime_activity=runtime_activity, name=name, broken=broken, skip=skip
+        runtime_activity = runtime_activity, name = name, broken = broken, skip = skip
     )
 end
 
 @testset "Bijectors integration tests" begin
     test_cases = TestCase[
         sum_b_binv_test_case(Bijectors.VecCorrBijector(), 3),
+        sum_b_binv_test_case(Bijectors.VecCorrBijector(), (1, 1)),
         sum_b_binv_test_case(Bijectors.VecCorrBijector(), 0),
         sum_b_binv_test_case(Bijectors.CorrBijector(), (3, 3)),
         sum_b_binv_test_case(Bijectors.CorrBijector(), (0, 0)),
@@ -141,16 +160,23 @@ end
         sum_b_binv_test_case(Bijectors.PDBijector(), (3, 3)),
         sum_b_binv_test_case(Bijectors.PDVecBijector(), 3),
         sum_b_binv_test_case(
-            Bijectors.Permute([
-                0 1 0;
-                1 0 0;
-                0 0 1
-            ]),
+            Bijectors.Permute(
+                [
+                    0 1 0;
+                    1 0 0;
+                    0 0 1
+                ]
+            ),
             (3, 3),
         ),
-        # TODO(mhauru) Both modes broken because of
-        # https://github.com/EnzymeAD/Enzyme.jl/issues/2035
-        sum_b_binv_test_case(Bijectors.PlanarLayer(3), (3, 3); broken=Both),
+        # NOTE(penelopeysm) This requires runtime activity on 1.11 reverse-mode, and 1.11
+        # forward-mode fails as this calls gemm! and runtime activity is not yet supported
+        # for BLAS calls. 1.10 works fine without runtime activity.
+        sum_b_binv_test_case(
+            Bijectors.PlanarLayer(3), (3, 3);
+            runtime_activity = ((v"1.10" <= VERSION < v"1.11" ? Neither : Reverse)),
+            broken = ((v"1.10" <= VERSION < v"1.11") ? Neither : Forward)
+        ),
         sum_b_binv_test_case(Bijectors.RadialLayer(3), 3),
         sum_b_binv_test_case(Bijectors.Reshape((2, 3), (3, 2)), (2, 3)),
         sum_b_binv_test_case(Bijectors.Scale(0.2), 3),
@@ -160,7 +186,29 @@ end
         sum_b_binv_test_case(Bijectors.TruncatedBijector(-0.2, 0.5), 3),
 
         # Below, some test cases that don't fit the sum_b_binv_test_case mold.
-
+        TestCase(
+            function (x)
+                return sum(Bijectors.PDVecBijector()(x * x' + I))
+            end,
+            randn(rng, 4, 4),
+            name = "PDVecBijector forward only",
+        ),
+        TestCase(
+            function (x)
+                binv = Bijectors.inverse(Bijectors.PDVecBijector())
+                return sum(cholesky(Hermitian(binv(x), :L)).L)
+            end,
+            Bijectors.PDVecBijector()((x -> x * x' + I)(randn(rng, 4, 4))),
+            name = "PDVecBijector inverse only + lower Cholesky",
+        ),
+        TestCase(
+            function (x)
+                binv = Bijectors.inverse(Bijectors.PDVecBijector())
+                return sum(cholesky(Hermitian(binv(x), :U)).U)
+            end,
+            Bijectors.PDVecBijector()((x -> x * x' + I)(randn(rng, 4, 4))),
+            name = "PDVecBijector inverse only + upper Cholesky",
+        ),
         TestCase(
             function (x)
                 b = Bijectors.RationalQuadraticSpline([-0.2, 0.1, 0.5], [-0.3, 0.3, 0.9], [1.0, 0.2, 1.0])
@@ -168,9 +216,8 @@ end
                 return sum(binv(b(x)))
             end,
             randn(rng);
-            name="RationalQuadraticSpline on scalar",
+            name = "RationalQuadraticSpline on scalar",
         ),
-
         TestCase(
             function (x)
                 b = Bijectors.OrderedBijector()
@@ -178,29 +225,47 @@ end
                 return sum(binv(b(x)))
             end,
             randn(rng, 7);
-            name="OrderedBijector",
+            name = "OrderedBijector",
         ),
-
         TestCase(
             function (x)
                 layer = Bijectors.PlanarLayer(x[1:2], x[3:4], x[5:5])
-                flow = Bijectors.transformed(Bijectors.MvNormal(zeros(2), LinearAlgebra.I), layer)
+                flow = Bijectors.transformed(Bijectors.MvNormal(zeros(2), I), layer)
                 x = x[6:7]
                 return Bijectors.logpdf(flow.dist, x) - Bijectors.logabsdetjac(flow.transform, x)
             end,
             randn(rng, 7);
-            name="PlanarLayer7"
+            name = "PlanarLayer7 forward"
         ),
-
         TestCase(
             function (x)
                 layer = Bijectors.PlanarLayer(x[1:2], x[3:4], x[5:5])
-                flow = Bijectors.transformed(Bijectors.MvNormal(zeros(2), LinearAlgebra.I), layer)
+                flow = Bijectors.transformed(Bijectors.MvNormal(zeros(2), I), layer)
                 x = reshape(x[6:end], 2, :)
                 return sum(Bijectors.logpdf(flow.dist, x) - Bijectors.logabsdetjac(flow.transform, x))
             end,
             randn(rng, 11);
-            name="PlanarLayer11"
+            name = "PlanarLayer11 forward"
+        ),
+        TestCase(
+            function (x)
+                layer = Bijectors.PlanarLayer(x[1:2], x[3:4], x[5:5])
+                flow = Bijectors.transformed(Bijectors.MvNormal(zeros(2), I), Bijectors.inverse(layer))
+                x = x[6:7]
+                return Bijectors.logpdf(flow.dist, x) - Bijectors.logabsdetjac(flow.transform, x)
+            end,
+            randn(rng, 7);
+            name = "PlanarLayer7 inverse"
+        ),
+        TestCase(
+            function (x)
+                layer = Bijectors.PlanarLayer(x[1:2], x[3:4], x[5:5])
+                flow = Bijectors.transformed(Bijectors.MvNormal(zeros(2), I), Bijectors.inverse(layer))
+                x = reshape(x[6:end], 2, :)
+                return sum(Bijectors.logpdf(flow.dist, x) - Bijectors.logabsdetjac(flow.transform, x))
+            end,
+            randn(rng, 11);
+            name = "PlanarLayer11 inverse"
         ),
     ]
 
